// Cartoon shader - by guest(r)
// guest.r@gmail.com
//
// license: GNU-GPL


// filter params

half threshold = 0.270;  // effects dark lines (raise if too dark)
half brighten  = 0.225;  // effects bright contrast

// definitions and sampler init 

half4x4 World			: WORLD;
half4x4 View			: VIEW;
half4x4 Projection		: PROJECTION;
half4x4 WorldView		: WORLDVIEW;			
half4x4 ViewProjection		: VIEWPROJECTION;		
half4x4 WorldViewProjection	: WORLDVIEWPROJECTION;		

float2 ps  : TEXELSIZE = 1.0F/256.0F;


texture s_tex	: SOURCETEXTURE;
sampler	s_sam = sampler_state
{
 Texture   = (s_tex);
 MinFilter = LINEAR;
 MagFilter = LINEAR;
};


string name : NAME = "cartoon";
string combineTechique : COMBINETECHNIQUE =  "cartoon";


// vertex shader linkin' structure

struct VERTEX_STUFF1
{
  float4  coord : POSITION;
  float2  CT   : TEXCOORD0;
  float4  t1   : TEXCOORD1;
  float4  t2   : TEXCOORD2;
  float4  t3   : TEXCOORD3;
  float4  t4   : TEXCOORD4;
};


// **VS**

VERTEX_STUFF1 S_VERTEX (float3 p : POSITION, float2 tc : TEXCOORD0)
{
  VERTEX_STUFF1 OUT = (VERTEX_STUFF1)0;
  
  float dx = ps.x*0.5;
  float dy = ps.y*0.5;

  OUT.coord = mul(float4(p,1),WorldViewProjection);
  OUT.CT = tc;
  OUT.t1.xy = tc + float2(-dx, 0);
  OUT.t2.xy = tc + float2( dx, 0);
  OUT.t3.xy = tc + float2( 0,-dy);
  OUT.t4.xy = tc + float2( 0, dy);
  OUT.t1.zw = tc + float2(-dx,-dy);
  OUT.t2.zw = tc + float2(-dx, dy);
  OUT.t3.zw = tc + float2( dx,-dy);
  OUT.t4.zw = tc + float2( dx, dy);

  return OUT;
}

// **PS**

float4 S_FRAGMENT ( in VERTEX_STUFF1 VAR ) : COLOR
{
   half3 c00 = tex2D(s_sam, VAR.t1.zw).xyz; 
   half3 c10 = tex2D(s_sam, VAR.t3.xy).xyz; 
   half3 c20 = tex2D(s_sam, VAR.t3.zw).xyz; 
   half3 c01 = tex2D(s_sam, VAR.t1.xy).xyz; 
   half3 c11 = tex2D(s_sam, VAR.CT).xyz; 
   half3 c21 = tex2D(s_sam, VAR.t2.xy).xyz; 
   half3 c02 = tex2D(s_sam, VAR.t2.zw).xyz; 
   half3 c12 = tex2D(s_sam, VAR.t4.xy).xyz; 
   half3 c22 = tex2D(s_sam, VAR.t4.zw).xyz; 

   half3 dt = half3(1.0,1.0,1.0);

   half d1=dot(abs(c00-c22),dt);
   half d2=dot(abs(c20-c02),dt);
   half hl=dot(abs(c01-c21),dt);
   half vl=dot(abs(c10-c12),dt);

   half d = 0.5*(d1+d2+hl+vl)/(dot(c11,dt)+0.15); d = min(1+brighten,1+threshold-d) ;

   return half4 (d*c11,1);
}


technique cartoon
{
   pass P0
   {
     VertexShader = compile vs_2_0 S_VERTEX();
     PixelShader  = compile ps_2_0 S_FRAGMENT();
   }  
}
